# =====================
# Main file for Place Shapes Toolbar
# =====================


require 'sketchup'


# =====================


module AF_Extensions

  module AF_Indoor


    # =====================

  
    # General variables
    @name = "TC-室内小助手1.3.0"
    @extname = "AF_Indoor"
    
    @extdir = File.dirname(__FILE__)   
    @sfolder = File.join(@extdir, "shapes_ip")
    @ifolder = File.join(@extdir, "icons")    
  
    # Set base unit for shapes from stored value
    @unit = Sketchup.read_default @extname, "unit", "foot"

    # @skps = Dir.entries(@sfolder)
    # UI.messagebox @skps


    # =====================

    
    def self.place_me(name)
      
      # Definition file to load
      mod = Sketchup.active_model
      skp = File.join(@sfolder, name+".skp")
      
      # Load defnition and scale it
      dname = name+"-"+@unit
      
      if !mod.definitions[dname]
      
          # Scaling factor based on unit
          case @unit
          when "inch"
            scale = 1.0 / 12
          when "mm"
            scale = 1.0 / 12 / 2.54 / 10
          when "cm"
            scale = 1.0 / 12 / 2.54
          when "m"
            scale = 1.0 / 12 / 2.54 * 100
          else
            scale = 1.0
          end
          t = Geom::Transformation.new scale          
          
          # Load definition and give it unit-based name
          defs = mod.definitions
          d = defs.load skp
          d.name = dname
          
          # Scale definition entities accordingly
          ents = []
          d.entities.each { |e|
            ents.push e
          }
          d.entities.transform_entities(t,ents)
          
      end
      
      c = mod.definitions[dname]
      i = mod.place_component c, true
    
    end


    # =====================


    def self.select_unit
    
      prompts = ["Base unit"]
      defaults = [@unit]
      lists = ["inch|foot|mm|cm|m"]
      res = UI.inputbox(prompts, defaults, lists, "Set shape base unit")
      @unit = res[0]
      Sketchup.write_default @extname, "unit", @unit

    end
    

    # ============================
  
  
   def self.clicka
	  
	  Sketchup.send_action 21098
	  
	  end # show_help
	  
	  
	  
	  def self.clica
	  definition = Sketchup.active_model.selection.first.definition
	  behavior = definition.behavior
	  behavior.no_scale_mask = 120
	  result = Sketchup.send_action "selectScaleTool:"
	  end
	  
	  
	  
	  def self.click
	  Sketchup.active_model.select_tool(DCInteractTool.new($dc_observers))
	  end
	


	  def self.web
	  
	   UI.openURL("http://www.afu321.cn/")
	  
	  end # show_help


	  
	   def self.options
	  
	   if $dc_observers.get_latest_class.configure_dialog_is_visible
	  $dc_observers.get_latest_class.close_configure_dialog
	  else
	  $dc_observers.get_latest_class.show_configure_dialog
	  end
	  Sketchup.send_action("selectSelectionTool:")
	  
	  end # show_help
    # =====================
    
    
    # Load extension at startup and add menu items
    if !file_loaded?(__FILE__)
    
      # Add to the Draw menu and create a toolbar    
      menu = UI.menu("Draw").add_submenu(@name)
      toolbar = UI::Toolbar.new @name    
      
      # All shapes to load in this order
      shapes = [""]
      
      # Add them all to menu and toolbar
      shapes.each { |s|
      
        cmd = UI::Command.new(s.capitalize) { self.place_me(s) }
        #cmd.small_icon = File.join(@ifolder, s + "_sm.png")
        #cmd.large_icon = File.join(@ifolder, s + "_l.png")
        cmd.tooltip = s.capitalize
        #cmd.status_bar_text = "Place " + s + " (multiple)"
        #menu.add_item cmd
        #toolbar.add_item cmd    
      
      }
      
      #menu.add_separator
      #toolbar.add_separator
      
      # Add the unit selector
      s = "绘制墙面"
      cmd = UI::Command.new(s.capitalize) {Sketchup.active_model.select_tool Indoor_Muro.new}
      cmd.small_icon = File.join(@ifolder, "Muroyc_sm.png")
      cmd.large_icon = File.join(@ifolder, "Muroyc_sm.png")
      cmd.tooltip = s.capitalize
      cmd.status_bar_text = "绘制墙面"
      menu.add_item cmd
      toolbar.add_item cmd   

		 s = "墙面偏移"
      cmd = UI::Command.new(s.capitalize) {Sketchup.active_model.select_tool Indoor_Murop.new}
      cmd.small_icon = File.join(@ifolder, "MuroP_sm.png")
      cmd.large_icon = File.join(@ifolder, "MuroP_sm.png")
      cmd.tooltip = s.capitalize
      cmd.status_bar_text = "墙面偏移"
      menu.add_item cmd
      toolbar.add_item cmd  

	   s = "墙面延长"
      cmd = UI::Command.new(s.capitalize) {Sketchup.active_model.select_tool Indoor_Estiramuro.new}
      cmd.small_icon = File.join(@ifolder, "yc_sm.png")
      cmd.large_icon = File.join(@ifolder, "yc_sm.png")
      cmd.tooltip = s.capitalize
      cmd.status_bar_text = "墙面延长"
      menu.add_item cmd
      toolbar.add_item cmd  
	  
	  
		 s = "生成墙体"
      cmd = UI::Command.new(s.capitalize) { Sketchup.active_model.select_tool Indoor_TresDimensiones.new }
      cmd.small_icon = File.join(@ifolder, "3D_sm.png")
      cmd.large_icon = File.join(@ifolder, "3D_sm.png")
      cmd.tooltip = s.capitalize
      cmd.status_bar_text = "生成立体的墙"
      menu.add_item cmd
      toolbar.add_item cmd  
	 
	 
	  s = "绘制梁柱"
      cmd = UI::Command.new(s.capitalize) { Sketchup.active_model.select_tool AFURotatedBox.new }
      cmd.small_icon = File.join(@ifolder, "3D3_sm.png")
      cmd.large_icon = File.join(@ifolder, "3D3_sm.png")
      cmd.tooltip = s.capitalize
      cmd.status_bar_text = "绘制梁柱"
      menu.add_item cmd
      toolbar.add_item cmd 
	 
	 
	  toolbar.add_separator
	  ######################################
	  
	  s = "实体穿透"
      cmd = UI::Command.new(s.capitalize) { Sketchup.active_model.select_tool AFU_HoleOnSolid.new( true ) }
      cmd.small_icon = File.join(@ifolder, "boxC_l.png")
      cmd.large_icon = File.join(@ifolder, "boxC_l.png")
      cmd.tooltip = s.capitalize
      cmd.status_bar_text = "实体穿透"
      menu.add_item cmd
      toolbar.add_item cmd 
	  
	  
	  
	  s = "跟随开口"
      cmd = UI::Command.new(s.capitalize) { TIG::HolePunchTool.new() }
      cmd.small_icon = File.join(@ifolder, "box_l.png")
      cmd.large_icon = File.join(@ifolder, "box_l.png")
      cmd.tooltip = s.capitalize
      cmd.status_bar_text = "跟随开口"
      menu.add_item cmd
      toolbar.add_item cmd 
	  
	
	 
	  s = "打开模型"
      cmd = UI::Command.new(s.capitalize) { self.clicka}
      cmd.small_icon = File.join(@ifolder, "as6.png")
      cmd.large_icon = File.join(@ifolder, "as6.png")
      cmd.tooltip = s.capitalize
      cmd.status_bar_text = "打开模型"
      menu.add_item cmd
      toolbar.add_item cmd 
	
	
	toolbar.add_separator
	
	  
	  
	  s = "资源/下载"
      cmd = UI::Command.new(s.capitalize) { self.web}
      cmd.small_icon = File.join(@ifolder, "as8.png")
      cmd.large_icon = File.join(@ifolder, "as8.png")
      cmd.tooltip = s.capitalize
      cmd.status_bar_text = "资源/更新"
      menu.add_item cmd
      toolbar.add_item cmd	  
	  
	  
	  
	  s = "组件选项"
      cmd = UI::Command.new(s.capitalize) { self.options}
      cmd.small_icon = File.join(@ifolder, "configurator_tool.png")
      cmd.large_icon = File.join(@ifolder, "configurator_tool.png")
      cmd.tooltip = s.capitalize
      cmd.status_bar_text = "组件选项"
      menu.add_item cmd
      toolbar.add_item cmd 
	  
	  
	  #s = "对象拉伸"
      #cmd = UI::Command.new(s.capitalize) { self.clica }
      #cmd.small_icon = File.join(@ifolder, "unit_sm.png")
      #cmd.large_icon = File.join(@ifolder, "unit_l.png")
      #cmd.tooltip = s.capitalize
      #cmd.status_bar_text = "对象拉伸"
      #menu.add_item cmd
      #toolbar.add_item cmd 
	  
	  
	  s = "组件交互"
      cmd = UI::Command.new(s.capitalize) { self.click}
      cmd.small_icon = File.join(@ifolder, "cursor_interact_tool.png")
      cmd.large_icon = File.join(@ifolder, "cursor_interact_tool.png")
      cmd.tooltip = s.capitalize
      cmd.status_bar_text = "组件交互"
      menu.add_item cmd
      toolbar.add_item cmd 
	  
	  
	  
	  
	  
	  
	  #toolbar.add_separator
	  
	  
	 


	  
      
      # And a link to get help
      #menu.add_item ("Help") { self.show_help }
  
      # Don't forget to show the toolbar
      toolbar.show
      
      # Let Ruby know we have loaded this file
      file_loaded(__FILE__)
    
    end # if  


    # =====================

  
  end # AF_Indoor
  
end # Extensions


# =====================
