#------------------------------------------------------------------------------------------------
# Permission to use, copy, modify, and distribute this software for 
# any purpose and without fee is hereby granted.
#------------------------------------------------------------------------------------------------
# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#------------------------------------------------------------------------------------------------
# IT WAS CREATED AND TESTED IN A WINDOWS ENVIRONMENT ONLY AND MAY NOT FUNCTION ON A MAC.
#------------------------------------------------------------------------------------------------
# THIS PLUGIN IS ANOTHER FIGMENT OF MY IMAGENATION AND IS NOT BASED ON OR COMPLY WITH ANYTHING! 
#------------------------------------------------------------------------------------------------
#    Name:	Rotated Box
#      By:	sdmitch
#   Usage:	Creates a rotated "solid" box on the plane defined by points 1,2,and 3.
#		 Note:	This is a modified version of Rotated Rectangle with the 3rd dimension added.
#						The default is to create groups, use Ctrl key or Alt on MAC to toggle group creation.
#    Date:	Jun 2012
#------------------------------------------------------------------------------------------------
#
require 'sketchup'
#


class AFURotatedBox

	def initialize
		@mod = Sketchup.active_model
		@ent = @mod.active_entities
		@ip = Sketchup::InputPoint.new
		@ip1 = Sketchup::InputPoint.new
		@guide_line = nil
		@group_box = true
		case RUBY_PLATFORM
			when /mswin/ ; @toggle = '(Ctrl)'
			when /darwin/ ; @toggle = '(Alt)'
		end
		self.reset
	end

	def reset
		@pts = []
		@state = 0
		@ip1.clear
		@height = 0
		@drawn = false
		if(@guide_line!=nil)
			Sketchup.active_model.abort_operation # in case operation aborted.
			@guide_line=nil
		end
		@prompt = "绘制梁柱, #{@toggle}=成组（Ctrl切换）:#{@group_box ? 'Off' : 'On'}, 点击下一点."
	end

	def onMouseMove(flags, x, y, view)
		self.set_current_point(x, y, view)
		Sketchup::set_status_text @prompt, SB_PROMPT
		Sketchup::set_status_text @vcb_label, SB_VCB_LABEL
		Sketchup::set_status_text @vcb_value, SB_VCB_VALUE
	end

	def onLButtonDown(flags, x, y, view)
		self.set_current_point(x, y, view)
		self.increment_state
		view.lock_inference
	end

	def set_current_point(x, y, view)
		if( !@ip.pick(view, x, y, @ip1) )
			return false
		end
		need_draw = true
		view.tooltip = @ip.tooltip	# Set tooltip to be displayed.
		case @state# Compute points
			when 0
				@pts[0] = @ip.position
				need_draw = @ip.display? || @drawn
			when 1
				@pts[1] = @ip.position
				@width = @pts[0].distance @pts[1]
				@vcb_label = "Width:"
				@vcb_value = @width.to_s
			when 2
				pt1 = @ip.position
				pt2 = pt1.project_to_line @pts
				vec = pt1 - pt2
				@depth = vec.length
				if( @depth > 0 )
					square_point = pt2.offset(vec, @width)# test for a square
					if( view.pick_helper.test_point(square_point, x, y) )
						@depth = @width
						@pts[2] = @pts[1].offset(vec, @depth)
						@pts[3] = @pts[0].offset(vec, @depth)
						view.tooltip = "Square"
					else
						@pts[2] = @pts[1].offset(vec)
						@pts[3] = @pts[0].offset(vec)
					end
				else
					@pts[2] = @pts[1]
					@pts[3] = @pts[0]
				end
				@vcb_label = "Depth:"
				@vcb_value = @depth.to_s
				vc=(@pts[2]-@pts[0]);vc.length=vc.length/2;@center=@pts[0]+vc
			when 3
				pt1 = @ip.position
				pln = Geom.fit_plane_to_points @pts[0],@pts[1],@pts[2]
				pt2 = pt1.project_to_plane pln
				@vec = pt1 - pt2; 
				if @vec.length > 0.0
					@height = @vec.length
					for i in 0..3
						@pts[i+4] = @pts[i].offset(@vec)
					end
				end
				@vcb_label = "Height:"
				@vcb_value = @height.to_s
		 end
		view.invalidate if need_draw
	end

	def increment_state
		@state += 1
		case @state
			when 1
				@ip1.copy! @ip
				@prompt = "设定 width, #{@toggle}=成组（Ctrl切换）:#{@group_box ? 'Off' : 'On'}, 点击下一点."
			when 2
				@ip1.clear
				@prompt = "设定 depth, #{@toggle}=成组（Ctrl切换）:#{@group_box ? 'Off' : 'On'}, 点击下一点."
			when 3
				@ip1.clear
				v1=(@pts[1]-@pts[2])
				v2=(@pts[3]-@pts[2])
				@normal= v1.cross(v2)
				@mod.start_operation "Rotated Box"
				@guide_line = @ent.add_cline(@ip.position,@normal)
				@prompt = "设定height, #{@toggle}=成组（Ctrl切换）:#{@group_box ? 'Off' : 'On'}, 点击下一点."
			when 4
				# check for zero height
				if( @height != 0.0 )
					@guide_line.erase!
					@group_box ? (grp=@ent.add_group;gent=grp.entities) : gent=@ent
					if @group_box
					# code from Rotated Rectangel Tool modified by Chris Phillips
						make_box(grp,@width,@depth,@height)
						v1=(@pts[0]-@pts[1]);v2=(@pts[0]-@pts[3]);v3=(@pts[4]-@pts[0])
						v1.reverse! if(v1.cross(v2).dot(v3)<0)
						xform=Geom::Transformation.new(v1,v2,v3,@center)*Geom::Transformation.new([0,0,@height/2])
						grp.transform!(xform);#orient the group axis to match the group side directions
					# thanks Chris
					else
						face=gent.add_face(@pts[0..3])
						@height = -@height if @vec.angle_between(face.normal) > Math::PI/2
						face.pushpull @height
					end
					@mod.commit_operation
					self.reset
				else
					UI.beep; puts "Zero height"
				end
		end
	end
	
	def make_box(g,w,d,h)
		# modified code of
		pts = [[-(w/2),-(d/2),-(h/2)],[w/2,-(d/2),-(h/2)],[w/2,d/2,-(h/2)],[-(w/2),d/2,-(h/2)],[-(w/2),-(d/2),-(h/2)]]
		base = g.entities.add_face pts
		h = -h if base.normal.dot(Z_AXIS) < 0.0
		base.pushpull h
	end
	
	def onRButtonDown(flag, x, y, view)
		view.invalidate if @drawn
		self.reset
	end

	def onCancel(flag,view)
		Sketchup.send_action "selectSelectionTool:"
		view.invalidate
	end

	def onUserText(text, view)
		begin
			value = text.to_l
		rescue
			UI.beep# Error parsing the text
			value = nil
			Sketchup::set_status_text "", SB_VCB_VALUE
		end
		return if !value
		case @state
			when 1
				vec = @pts[1] - @pts[0]
				if( vec.length > 0.0 )
					@width = value# update the width
					@pts[1] = @pts[0].offset(vec,@width)
					view.invalidate
					self.increment_state
				end
			when 2
				vec = @pts[3] - @pts[0]
				if( vec.length > 0.0 )
					@depth = value# update the depth
					@pts[2] = @pts[1].offset(vec,@depth)
					@pts[3] = @pts[0].offset(vec,@depth)
					self.increment_state
				end
			when 3
				if value > 0.0
					@height = value# update the height
					self.increment_state
				end
		end
	end

	def draw(view)
		@drawn = false
		if( @ip.valid? && @ip.display? )
			@ip.draw(view)# Show the current input point
			@drawn = true
		end
		if @state == 1# draw line from the start to current point
			view.set_color_from_line(@ip1, @ip)
			inference_locked = view.inference_locked?
			view.line_width = 3 if inference_locked
			view.draw(GL_LINE_STRIP, @pts[0], @pts[1])
			view.line_width = 1 if inference_locked
		elsif @state == 2	# draw the base rectangle
			view.drawing_color = "blue"
			view.draw(GL_LINE_LOOP, @pts)
			if @depth==@width
				view.line_stipple = "-"
				view.draw_line @pts[0],@pts[2]
			end
		elsif @state == 3	# draw the box
			view.drawing_color = "red"
			view.draw GL_LINE_LOOP, @pts[0..3]
			if  @height > 0
				view.draw GL_LINE_LOOP, @pts[4..7]
				for i in 0..3
					view.draw GL_LINES,@pts[i],@pts[i+4]
				end
			end
		 end
	end

	def onKeyDown(key, rpt, flags, view)
		if( key == CONSTRAIN_MODIFIER_KEY && rpt == 1 )
			@shift_down_time = Time.now
			if( view.inference_locked? )
				view.lock_inference
			elsif( @state == 0 )
				view.lock_inference @ip
			elsif( @state == 1 )
				view.lock_inference @ip, @ip1
			end
		elsif  key == VK_CONTROL || key == VK_ALT# Ctrl on PC & Alt on MAC
			@group_box = !@group_box
			case @state
				when 0 ; @prompt = "绘制梁柱, #{@toggle}=成组（Ctrl切换）:#{@group_box ? 'Off' : 'On'}, Esc 退出绘制."
				when 1 ; @prompt = "设定 width, #{@toggle}=成组（Ctrl切换）:#{@group_box ? 'Off' : 'On'}, 点击下一点."
				when 2 ; @prompt = "设定 depth, #{@toggle}=成组（Ctrl切换）:#{@group_box ? 'Off' : 'On'}, 点击下一点."
				when 3 ; @prompt = "设定 height, #{@toggle}=成组（Ctrl切换）:#{@group_box ? 'Off' : 'On'}, 点击下一点."
			end
			Sketchup::set_status_text @prompt
		end
	end

	def onKeyUp(key, rpt, flags, view)
		if( key == CONSTRAIN_MODIFIER_KEY &&
			view.inference_locked? &&
			(Time.now - @shift_down_time) > 0.5 )
			view.lock_inference
		end
	end

end # of class AFURotatedBox
